diff -ru a/include/vlc/libvlc_events.h b/include/vlc/libvlc_events.h
--- a/include/vlc/libvlc_events.h	2008-11-17 00:04:52.000000000 +0100
+++ b/include/vlc/libvlc_events.h	2008-11-20 14:19:53.000000000 +0100
@@ -80,6 +80,7 @@
     libvlc_MediaListPlayerPlayed,
     libvlc_MediaListPlayerNextItemSet,
     libvlc_MediaListPlayerStopped,
+    libvlc_MediaListPlayerEndReached,
 
     libvlc_MediaDiscovererStarted,
     libvlc_MediaDiscovererEnded,
diff -ru a/src/control/core.c b/src/control/core.c
--- a/src/control/core.c	2008-11-17 00:04:59.000000000 +0100
+++ b/src/control/core.c	2008-11-21 23:00:09.000000000 +0100
@@ -43,6 +43,9 @@
 
 void libvlc_exception_clear( libvlc_exception_t *p_exception )
 {
+    if ( !p_exception )
+        return;
+
     if( p_exception->psz_message != nomemstr )
         free( p_exception->psz_message );
     p_exception->psz_message = NULL;
diff -ru a/src/control/media_list_path.h b/src/control/media_list_path.h
--- a/src/control/media_list_path.h	2008-11-17 00:04:59.000000000 +0100
+++ b/src/control/media_list_path.h	2008-11-21 19:46:20.000000000 +0100
@@ -75,7 +75,7 @@
     libvlc_media_list_path_t ret;
     int old_deepness = libvlc_media_list_path_deepness( path );
     ret = malloc( sizeof(int)*(old_deepness+2) );
-    memcpy( ret, path, sizeof(int)*(old_deepness+2) );
+    memcpy( ret, path, sizeof(int)*(old_deepness) );
     ret[old_deepness] = index;
     ret[old_deepness+1] = -1;
     return ret;
@@ -149,7 +149,6 @@
     for( i = 0; path[i] != -1; i++ )
     {
         p_md = libvlc_media_list_item_at_index( p_current_mlist, path[i], NULL );
-
         if( p_current_mlist != p_mlist )
             libvlc_media_list_release( p_current_mlist );
 
@@ -185,10 +184,12 @@
             libvlc_media_list_release( p_current_mlist );
 
         if( path[i+1] == -1 )
+        {
+            libvlc_media_list_retain( p_current_mlist );
             return p_current_mlist;
+        }
 
         p_md = libvlc_media_list_item_at_index( p_current_mlist, path[i], NULL );
-
         p_current_mlist = libvlc_media_subitems( p_md, NULL );
         libvlc_media_release( p_md );
  
diff -ru a/src/control/media_list_player.c b/src/control/media_list_player.c
--- a/src/control/media_list_player.c	2008-11-21 23:57:07.000000000 +0100
+++ b/src/control/media_list_player.c	2008-11-22 00:00:29.000000000 +0100
@@ -31,65 +31,133 @@
 /**************************************************************************
  *       get_next_index (private)
  *
- * Simple next item fetcher.
+ * next item fetcher.
+ *  - if current item or first item is a node, go to its deepest item, play it
+ *  - if not a node and a first item, play it
+ *  - if not a node and not first item, go to next item 
+ *  - if next item is a node, go to its deepest item, play it
+ *  - if next item not a node, play it
  **************************************************************************/
 static libvlc_media_list_path_t
 get_next_path( libvlc_media_list_player_t * p_mlp )
 {
-    /* We are entered with libvlc_media_list_lock( p_mlp->p_list ) */
+    /* We are entered with libvlc_media_list_lock( p_mlp->p_mlist ) */
     libvlc_media_list_path_t ret;
     libvlc_media_list_t * p_parent_of_playing_item;
     libvlc_media_list_t * p_sublist_of_playing_item;
 
+    bool b_first_playing_item;
     if ( !p_mlp->current_playing_item_path )
     {
-        if( !libvlc_media_list_count( p_mlp->p_mlist, NULL ) )
+        if ( libvlc_media_list_count( p_mlp->p_mlist, NULL ) > 0 )
+        {
+            b_first_playing_item = true;
+            ret = libvlc_media_list_path_with_root_index(0);
+        }
+        else
             return NULL;
-        return libvlc_media_list_path_with_root_index(0);
     }
-    
-    p_sublist_of_playing_item = libvlc_media_list_sublist_at_path(
-                            p_mlp->p_mlist,
-                            p_mlp->current_playing_item_path );
- 
-    /* If item just gained a sublist just play it */
-    if( p_sublist_of_playing_item )
+    else
     {
-        libvlc_media_list_release( p_sublist_of_playing_item );
-        return libvlc_media_list_path_copy_by_appending( p_mlp->current_playing_item_path, 0 );
+        b_first_playing_item = false;
+        ret = libvlc_media_list_path_copy( p_mlp->current_playing_item_path );
     }
 
-    /* Try to catch next element */
-    p_parent_of_playing_item = libvlc_media_list_parentlist_at_path(
-                            p_mlp->p_mlist,
-                            p_mlp->current_playing_item_path );
-
+    /**
+     * If item has got a sublist just play its deepest item 
+     */
+    bool b_sublist_of_playing_item = false;
+    while(1)
+    {
+        p_sublist_of_playing_item = libvlc_media_list_sublist_at_path( p_mlp->p_mlist, ret );
+        if ( !p_sublist_of_playing_item )
+            break;
+ 
+        libvlc_media_list_path_t ret_tmp;
+        ret_tmp = libvlc_media_list_path_copy_by_appending( ret, 0 );
+        libvlc_media_list_release( p_sublist_of_playing_item );
+        free( ret );
+        ret = ret_tmp;
+        b_sublist_of_playing_item = true;
+    }
+    if ( b_first_playing_item || b_sublist_of_playing_item )
+        return ret;
+
+    /**
+     * No sublist, then try to catch next element 
+     * */
     int deepness = libvlc_media_list_path_deepness( p_mlp->current_playing_item_path );
-    if( deepness < 1 || !p_parent_of_playing_item )
-        return NULL;
 
-    ret = libvlc_media_list_path_copy( p_mlp->current_playing_item_path );
-
-    while( ret[deepness-1] >= libvlc_media_list_count( p_parent_of_playing_item, NULL ) )
+    while ( deepness > 0 )
     {
-        deepness--;
-        if( deepness <= 0 )
+        if ( deepness > 1 )
+            p_parent_of_playing_item = 
+                libvlc_media_list_parentlist_at_path( p_mlp->p_mlist, ret );
+        else
+            p_parent_of_playing_item = p_mlp->p_mlist;
+
+        if ( !p_parent_of_playing_item )
         {
             free( ret );
-            libvlc_media_list_release( p_parent_of_playing_item );
             return NULL;
         }
-        ret[deepness] = -1;
+
         ret[deepness-1]++;
-        p_parent_of_playing_item  = libvlc_media_list_parentlist_at_path(
-                                        p_mlp->p_mlist,
-                                        ret );
+        if ( ret[deepness-1] < libvlc_media_list_count( p_parent_of_playing_item, NULL ) )
+        {
+            /* found a suitable item */
+            if ( deepness > 1 )
+                libvlc_media_list_release( p_parent_of_playing_item );
+            break;
+        }
+
+        if ( deepness > 1 )
+            libvlc_media_list_release( p_parent_of_playing_item );
+        deepness--;
+        ret[deepness] = -1;
+    } 
+
+    /* no more item (end of playlist reached) */
+    if ( deepness == 0 )
+    {
+        free( ret );
+        return NULL;
+    } 
+
+
+    /**
+     * If new item has got a sublist then go for its deepest item 
+     */
+    while(1)
+    {
+        p_sublist_of_playing_item = libvlc_media_list_sublist_at_path( p_mlp->p_mlist, ret );
+        if ( !p_sublist_of_playing_item )
+            break;
+ 
+        libvlc_media_list_path_t ret_tmp;
+        ret_tmp = libvlc_media_list_path_copy_by_appending( ret, 0 );
+        libvlc_media_list_release( p_sublist_of_playing_item );
+        free( ret );
+        ret = ret_tmp;
     }
-    libvlc_media_list_release( p_parent_of_playing_item );
+
     return ret;
 }
 
 /**************************************************************************
+ *       media_player_reached_end_forked (private) 
+ **************************************************************************/
+static void *
+media_player_reached_end_forked( void * p_data )
+{
+    libvlc_media_list_player_t * p_mlp = p_data;
+
+    /* function called in a separate thread to allow release of previous
+     * input thread */
+    libvlc_media_list_player_next( p_mlp, NULL );
+}
+
+/**************************************************************************
  *       media_player_reached_end (private) (Event Callback)
  **************************************************************************/
 static void
@@ -115,7 +183,24 @@
     }
     libvlc_media_release( p_md );
     libvlc_media_release( p_current_md );
-    libvlc_media_list_player_next( p_mlp, NULL );
+
+    /**
+    * This callback is run by the input thread.
+    * Executing "libvlc_media_list_player_next" here leads to a deadlock issue.
+    * (wants to clean up the input thread to give room for the new input,
+    * and ends up with input thread waiting for its own death ! (vlc_join) )
+    * Running "libvlc_media_list_player_next" as a dedicated thread
+    * is a workaround.
+    */ 
+    // libvlc_media_list_player_next( p_mlp, NULL );
+
+    vlc_thread_t handle;
+    if( vlc_clone( &handle, media_player_reached_end_forked, p_mlp,
+                   VLC_THREAD_PRIORITY_LOW ) )
+    {
+        msg_Warn( p_mlp->p_libvlc_instance->p_libvlc_int,
+                  "media_list_player_reached_end: forking new thread failed" );
+    }
 }
 
 /**************************************************************************
@@ -230,16 +315,9 @@
         p_mlp->p_mi = libvlc_media_player_new_from_media(p_md, p_e);
     }
     
-    if( p_md->p_subitems && libvlc_media_list_count( p_md->p_subitems, NULL ) > 0 )
-    {
-        libvlc_media_t * p_submd;
-        p_submd = libvlc_media_list_item_at_index( p_md->p_subitems, 0, NULL );
-        libvlc_media_player_set_media( p_mlp->p_mi, p_submd, NULL );
-        libvlc_media_release( p_submd );
-    }
-    else
-        libvlc_media_player_set_media( p_mlp->p_mi, p_md, NULL );
+    libvlc_media_player_set_media( p_mlp->p_mi, p_md, NULL );
 //    wait_playing_state(); /* If we want to be synchronous */
+
     install_media_player_observer( p_mlp );
 
     vlc_mutex_unlock( &p_mlp->object_lock );
@@ -267,6 +345,7 @@
     p_mlp->current_playing_item_path = NULL;
     p_mlp->p_mi = NULL;
     p_mlp->p_mlist = NULL;
+    p_mlp->p_libvlc_instance = p_instance;
     vlc_mutex_init( &p_mlp->object_lock );
     p_mlp->p_event_manager = libvlc_event_manager_new( p_mlp,
                                                        p_instance,
@@ -475,7 +554,12 @@
     if( !path )
     {
         libvlc_media_list_unlock( p_mlp->p_mlist );
-        libvlc_exception_raise( p_e, "No more element to play" );
+
+        /* Send the EndReached item event */
+        libvlc_event_t event;
+        event.type = libvlc_MediaListPlayerEndReached;
+        libvlc_event_send( p_mlp->p_event_manager, &event);
+
         libvlc_media_list_player_stop( p_mlp, p_e );
         return;
     }
